@@ -33,7 +33,12 @@
 #endif
 
 #include <dbus/dbus-glib.h>
+#include <stdlib.h>
 #include <string.h>
+#include <sys/types.h>
+#include <unistd.h>
+#include <glibtop/proclist.h>
+#include <glibtop/procstate.h>
 
 #include <libnautilus-extension/nautilus-file-info.h>
 
@@ -70,13 +75,8 @@ static void     split_mimetype( const gchar *mimetype, gchar **group, gchar **su
 static gboolean is_candidate_for_basenames( const NAIContext *object, guint target, GList *files );
 static gboolean is_candidate_for_selection_count( const NAIContext *object, guint target, GList *files );
 static gboolean is_candidate_for_schemes( const NAIContext *object, guint target, GList *files );
-#if 0
-static gboolean validate_schemes( GSList *object_schemes, NASelectedInfo *iter );
-#endif
+static gboolean is_compatible_scheme( const gchar *pattern, const gchar *scheme );
 static gboolean is_candidate_for_folders( const NAIContext *object, guint target, GList *files );
-#if 0
-static gboolean is_current_folder_inside( const NAIContext *object, NASelectedInfo *current_folder );
-#endif
 static gboolean is_candidate_for_capabilities( const NAIContext *object, guint target, GList *files );
 
 static gboolean is_valid_basenames( const NAIContext *object );
@@ -168,49 +168,49 @@ interface_base_finalize( NAIContextInterface *klass )
 
 /**
  * na_icontext_is_candidate:
- * @object: a #NAIContext to be checked.
+ * @context: a #NAIContext to be checked.
  * @target: the current target.
- * @files: the currently selected items, as a #GList of #NASelectedInfo items.
+ * @selection: the currently selected items, as a #GList of #NASelectedInfo items.
  *
  * Determines if the given object may be candidate to be displayed in
  * the Nautilus context menu, depending of the list of currently selected
  * items.
  *
  * This function is called by nautilus-actions::build_nautilus_menus()
- * for each item found in NAPivot items list, and, when this an action,
+ * for each item found in #NAPivot items list, and, when this an action,
  * for each profile of this action.
  *
- * Returns: %TRUE if this object succeeds to all tests and is so a
+ * Returns: %TRUE if this @context succeeds to all tests and is so a
  * valid candidate to be displayed in Nautilus context menu, %FALSE
  * else.
  */
 gboolean
-na_icontext_is_candidate( const NAIContext *object, guint target, GList *files )
+na_icontext_is_candidate( const NAIContext *context, guint target, GList *selection )
 {
 	static const gchar *thisfn = "na_icontext_is_candidate";
 	gboolean is_candidate;
 
-	g_return_val_if_fail( NA_IS_ICONTEXT( object ), FALSE );
+	g_return_val_if_fail( NA_IS_ICONTEXT( context ), FALSE );
 
-	g_debug( "%s: object=%p (%s), target=%d, files=%p (count=%d)",
-			thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ), target, (void * ) files, g_list_length( files ));
+	g_debug( "%s: object=%p (%s), target=%d, selection=%p (count=%d)",
+			thisfn, ( void * ) context, G_OBJECT_TYPE_NAME( context ), target, (void * ) selection, g_list_length( selection ));
 
-	is_candidate = v_is_candidate( NA_ICONTEXT( object ), target, files );
+	is_candidate = v_is_candidate( NA_ICONTEXT( context ), target, selection );
 
 	if( is_candidate ){
 		is_candidate =
-				is_candidate_for_target( object, target, files ) &&
-				is_candidate_for_show_in( object, target, files ) &&
-				is_candidate_for_try_exec( object, target, files ) &&
-				is_candidate_for_show_if_registered( object, target, files ) &&
-				is_candidate_for_show_if_true( object, target, files ) &&
-				is_candidate_for_show_if_running( object, target, files ) &&
-				is_candidate_for_mimetypes( object, target, files ) &&
-				is_candidate_for_basenames( object, target, files ) &&
-				is_candidate_for_selection_count( object, target, files ) &&
-				is_candidate_for_schemes( object, target, files ) &&
-				is_candidate_for_folders( object, target, files ) &&
-				is_candidate_for_capabilities( object, target, files );
+				is_candidate_for_target( context, target, selection ) &&
+				is_candidate_for_show_in( context, target, selection ) &&
+				is_candidate_for_try_exec( context, target, selection ) &&
+				is_candidate_for_show_if_registered( context, target, selection ) &&
+				is_candidate_for_show_if_true( context, target, selection ) &&
+				is_candidate_for_show_if_running( context, target, selection ) &&
+				is_candidate_for_mimetypes( context, target, selection ) &&
+				is_candidate_for_basenames( context, target, selection ) &&
+				is_candidate_for_selection_count( context, target, selection ) &&
+				is_candidate_for_schemes( context, target, selection ) &&
+				is_candidate_for_folders( context, target, selection ) &&
+				is_candidate_for_capabilities( context, target, selection );
 	}
 
 	return( is_candidate );
@@ -218,29 +218,29 @@ na_icontext_is_candidate( const NAIContext *object, guint target, GList *files )
 
 /**
  * na_icontext_is_valid:
- * @profile: the #NAObjectProfile to be checked.
+ * @context: the #NAObjectProfile to be checked.
  *
- * Returns: %TRUE if this profile is valid, %FALSE else.
+ * Returns: %TRUE if this @context is valid, %FALSE else.
  *
- * This function is part of NAIDuplicable::check_status() and is called
- * by NAIDuplicable objects which also implement NAIContext
+ * This function is part of #NAIDuplicable::check_status() and is called
+ * by #NAIDuplicable objects which also implement #NAIContext
  * interface. It so doesn't make sense of asking the object for its
  * validity status as it has already been checked before calling the
  * function.
  */
 gboolean
-na_icontext_is_valid( const NAIContext *object )
+na_icontext_is_valid( const NAIContext *context )
 {
 	gboolean is_valid;
 
-	g_return_val_if_fail( NA_IS_ICONTEXT( object ), FALSE );
+	g_return_val_if_fail( NA_IS_ICONTEXT( context ), FALSE );
 
 	is_valid =
-		is_valid_basenames( object ) &&
-		is_valid_mimetypes( object ) &&
-		is_valid_isfiledir( object ) &&
-		is_valid_schemes( object ) &&
-		is_valid_folders( object );
+		is_valid_basenames( context ) &&
+		is_valid_mimetypes( context ) &&
+		is_valid_isfiledir( context ) &&
+		is_valid_schemes( context ) &&
+		is_valid_folders( context );
 
 	return( is_valid );
 }
@@ -249,18 +249,18 @@ na_icontext_is_valid( const NAIContext *object )
  * na_icontext_is_all_mimetypes:
  * @context: the #NAIContext object to be checked.
  *
- * Returns: %TRUE if this context is valid for all mimetypes, %FALSE else.
+ * Returns: %TRUE if this @context is valid for all mimetypes, %FALSE else.
  */
 gboolean
-na_icontext_is_all_mimetypes( const NAIContext *object )
+na_icontext_is_all_mimetypes( const NAIContext *context )
 {
 	gboolean is_all;
 	GSList *mimetypes, *im;
 
-	g_return_val_if_fail( NA_IS_ICONTEXT( object ), FALSE );
+	g_return_val_if_fail( NA_IS_ICONTEXT( context ), FALSE );
 
 	is_all = TRUE;
-	mimetypes = na_object_get_mimetypes( object );
+	mimetypes = na_object_get_mimetypes( context );
 
 	for( im = mimetypes ; im ; im = im->next ){
 		if( !im->data || !strlen( im->data )){
@@ -295,11 +295,11 @@ na_icontext_read_done( NAIContext *context )
 
 /**
  * na_icontext_set_scheme:
- * @profile: the #NAIContext to be updated.
+ * @context: the #NAIContext to be updated.
  * @scheme: name of the scheme.
- * @selected: whether this scheme is candidate to this context.
+ * @selected: whether this scheme is candidate to this @context.
  *
- * Sets the status of a scheme relative to this context.
+ * Sets the status of a @scheme relative to this @context.
  */
 void
 na_icontext_set_scheme( NAIContext *context, const gchar *scheme, gboolean selected )
@@ -316,11 +316,11 @@ na_icontext_set_scheme( NAIContext *context, const gchar *scheme, gboolean selec
 
 /**
  * na_icontext_set_only_desktop:
- * @profile: the #NAIContext to be updated.
+ * @context: the #NAIContext to be updated.
  * @desktop: name of the desktop environment.
- * @selected: whether this desktop is candidate to this context.
+ * @selected: whether this @desktop is candidate to this @context.
  *
- * Sets the status of the desktop relative to this context for the OnlyShowIn list.
+ * Sets the status of the @desktop relative to this @context for the OnlyShowIn list.
  */
 void
 na_icontext_set_only_desktop( NAIContext *context, const gchar *desktop, gboolean selected )
@@ -337,11 +337,11 @@ na_icontext_set_only_desktop( NAIContext *context, const gchar *desktop, gboolea
 
 /**
  * na_icontext_set_only_desktop:
- * @profile: the #NAIContext to be updated.
+ * @context: the #NAIContext to be updated.
  * @desktop: name of the desktop environment.
- * @selected: whether this desktop is candidate to this context.
+ * @selected: whether this @desktop is candidate to this @context.
  *
- * Sets the status of the desktop relative to this context for the NotShowIn list.
+ * Sets the status of the @desktop relative to this @context for the NotShowIn list.
  */
 void
 na_icontext_set_not_desktop( NAIContext *context, const gchar *desktop, gboolean selected )
@@ -358,35 +358,35 @@ na_icontext_set_not_desktop( NAIContext *context, const gchar *desktop, gboolean
 
 /**
  * na_icontext_replace_folder:
- * @profile: the #NAIContext to be updated.
+ * @context: the #NAIContext to be updated.
  * @old: the old uri.
  * @new: the new uri.
  *
  * Replaces the @old URI by the @new one.
  */
 void
-na_icontext_replace_folder( NAIContext *profile, const gchar *old, const gchar *new )
+na_icontext_replace_folder( NAIContext *context, const gchar *old, const gchar *new )
 {
 	GSList *folders;
 
-	g_return_if_fail( NA_IS_ICONTEXT( profile ));
+	g_return_if_fail( NA_IS_ICONTEXT( context ));
 
-	folders = na_object_get_folders( profile );
+	folders = na_object_get_folders( context );
 	folders = na_core_utils_slist_remove_utf8( folders, old );
 	folders = g_slist_append( folders, ( gpointer ) g_strdup( new ));
-	na_object_set_folders( profile, folders );
+	na_object_set_folders( context, folders );
 	na_core_utils_slist_free( folders );
 }
 
 static gboolean
-v_is_candidate( NAIContext *object, guint target, GList *selection )
+v_is_candidate( NAIContext *context, guint target, GList *selection )
 {
 	gboolean is_candidate;
 
 	is_candidate = TRUE;
 
-	if( NA_ICONTEXT_GET_INTERFACE( object )->is_candidate ){
-		is_candidate = NA_ICONTEXT_GET_INTERFACE( object )->is_candidate( object, target, selection );
+	if( NA_ICONTEXT_GET_INTERFACE( context )->is_candidate ){
+		is_candidate = NA_ICONTEXT_GET_INTERFACE( context )->is_candidate( context, target, selection );
 	}
 
 	return( is_candidate );
@@ -575,12 +575,29 @@ is_candidate_for_show_if_running( const NAIContext *object, guint target, GList
 {
 	static const gchar *thisfn = "na_icontext_is_candidate_for_show_if_running";
 	gboolean ok = TRUE;
+	gchar *searched;
+	glibtop_proclist proclist;
+	glibtop_proc_state procstate;
+	pid_t *pid_list;
+	guint i;
 	gchar *running = na_object_get_show_if_running( object );
 
 	if( running && strlen( running )){
-		/* TODO: how to get the list of running processes ?
-		 * or how to known if a given process is currently running
-		 */
+		ok = FALSE;
+		searched = g_path_get_basename( running );
+		pid_list = glibtop_get_proclist( &proclist, GLIBTOP_KERN_PROC_ALL, 0 );
+
+		for( i=0 ; i<proclist.number && !ok ; ++i ){
+			glibtop_get_proc_state( &procstate, pid_list[i] );
+			/*g_debug( "%s: i=%d, cmd=%s", thisfn, i, procstate.cmd );*/
+			if( strcmp( procstate.cmd, searched ) == 0 ){
+				g_debug( "%s: i=%d, cmd=%s", thisfn, i, procstate.cmd );
+				ok = TRUE;
+			}
+		}
+
+		g_free( pid_list );
+		g_free( searched );
 	}
 
 	if( !ok ){
@@ -604,6 +621,14 @@ is_candidate_for_show_if_running( const NAIContext *object, guint target, GList
  * file mimetype must be compatible with at least one positive assertion
  * (they are ORed), while not being of any negative assertions (they are
  * ANDed)
+ *
+ * here, for each mimetype of the selection, do
+ * . match is FALSE while this mimetype has not matched a positive assertion
+ * . nomatch is FALSE while this mimetype has not matched a negative assertion
+ * we are going to check each mimetype filter
+ *  while we have not found a match among positive conditions (i.e. while !match)
+ *  we have to check all negative conditions to verify that the current
+ *  examined mimetype never match these
  */
 static gboolean
 is_candidate_for_mimetypes( const NAIContext *object, guint target, GList *files )
@@ -611,50 +636,42 @@ is_candidate_for_mimetypes( const NAIContext *object, guint target, GList *files
 	static const gchar *thisfn = "na_icontext_is_candidate_for_mimetypes";
 	gboolean ok = TRUE;
 	gboolean all = na_object_get_all_mimetypes( object );
-	GList *it;
 
 	g_debug( "%s: all=%s", thisfn, all ? "True":"False" );
 
 	if( !all ){
 		GSList *mimetypes = na_object_get_mimetypes( object );
 		GSList *im;
-		gboolean positive;
-		guint count_positive = 0;
-		guint count_compatible = 0;
+		GList *it;
 
-		for( im = mimetypes ; im && ok ; im = im->next ){
-			const gchar *imtype = ( const gchar * ) im->data;
-			positive = is_positive_assertion( imtype );
-			if( positive ){
-				count_positive += 1;
-			}
-			gchar *imgroup, *imsubgroup;
-			split_mimetype( imtype, &imgroup, &imsubgroup );
+		for( it = files ; it && ok ; it = it->next ){
+			gchar *ftype, *fgroup, *fsubgroup;
+			gboolean match, positive;
 
-			for( it = files ; it && ok ; it = it->next ){
-				gchar *ftype = na_selected_info_get_mime_type( NA_SELECTED_INFO( it->data ));
-				gboolean type_of = is_mimetype_of( ftype, imgroup, imsubgroup );
+			ftype = na_selected_info_get_mime_type( NA_SELECTED_INFO( it->data ));
+			split_mimetype( ftype, &fgroup, &fsubgroup );
+			match = FALSE;
 
-				if( positive ){
-					if( type_of ){
-						count_compatible += 1;
-					}
-				} else {
-					if( type_of ){
-						ok = FALSE;
-						break;
+			for( im = mimetypes ; im && ok ; im = im->next ){
+				const gchar *imtype = ( const gchar * ) im->data;
+				positive = is_positive_assertion( imtype );
+
+				if( !positive || !match ){
+					if( is_mimetype_of( positive ? imtype : imtype+1, fgroup, fsubgroup )){
+						if( positive ){
+							match = TRUE;
+						} else {
+							ok = FALSE;
+						}
 					}
 				}
-
-				g_free( ftype );
 			}
 
-			g_free( imgroup );
-			g_free( imsubgroup );
-		}
+			ok &= match;
 
-		if( count_positive > 0 && count_compatible == 0 ){
-			ok = FALSE;
+			g_free( fsubgroup );
+			g_free( fgroup );
+			g_free( ftype );
 		}
 
 		if( !ok ){
@@ -670,31 +687,35 @@ is_candidate_for_mimetypes( const NAIContext *object, guint target, GList *files
 }
 
 /*
- * does the file have a mimetype which is 'a sort of' specified one ?
+ * does the file fgroup/fsubgrop have a mimetype which is 'a sort of'
+ *  mimetype specified one ?
  * for example, "image/jpeg" is clearly a sort of "image/ *"
  * but how to check to see if "msword/xml" is a sort of "application/xml" ??
  */
 static gboolean
-is_mimetype_of( const gchar *file_type, const gchar *group, const gchar *subgroup )
+is_mimetype_of( const gchar *mimetype, const gchar *fgroup, const gchar *fsubgroup )
 {
 	gboolean is_type_of;
-	gchar *fgroup, *fsubgroup;
+	gchar *mgroup, *msubgroup;
 
-	split_mimetype( file_type, &fgroup, &fsubgroup );
+	split_mimetype( mimetype, &mgroup, &msubgroup );
 
-	is_type_of = ( strcmp( fgroup, group ) == 0 );
+	is_type_of = ( strcmp( fgroup, mgroup ) == 0 );
 	if( is_type_of ){
-		if( strcmp( subgroup, "*" )){
-			is_type_of = ( strcmp( fsubgroup, subgroup ) == 0 );
+		if( strcmp( msubgroup, "*" ) != 0 ){
+			is_type_of = ( strcmp( fsubgroup, msubgroup ) == 0 );
 		}
 	}
 
-	g_free( fgroup );
-	g_free( fsubgroup );
+	g_free( mgroup );
+	g_free( msubgroup );
 
 	return( is_type_of );
 }
 
+/*
+ * split the given mimetype to its group and subgroup components
+ */
 static void
 split_mimetype( const gchar *mimetype, gchar **group, gchar **subgroup )
 {
@@ -714,8 +735,44 @@ is_candidate_for_basenames( const NAIContext *object, guint target, GList *files
 	GSList *basenames = na_object_get_basenames( object );
 
 	if( basenames ){
-		gboolean matchcase = na_object_get_matchcase( object );
-		matchcase = FALSE;
+		if( strcmp( basenames->data, "*" ) != 0 || g_slist_length( basenames ) > 1 ){
+			gboolean matchcase = na_object_get_matchcase( object );
+			GSList *ib;
+			GList *it;
+
+			for( it = files ; it && ok ; it = it->next ){
+				gchar *pattern, *bname, *bname_utf8;
+				gboolean match, positive;
+
+				bname = na_selected_info_get_basename( NA_SELECTED_INFO( it->data ));
+				bname_utf8 = g_filename_to_utf8( bname, -1, NULL, NULL, NULL );
+				match = FALSE;
+
+				for( ib = basenames ; ib && ok ; ib = ib->next ){
+					pattern = matchcase ?
+						g_strdup(( gchar * ) ib->data ) :
+						g_ascii_strdown(( gchar * ) ib->data, strlen(( gchar * ) ib->data ));
+					positive = is_positive_assertion( pattern );
+
+					if( !positive || !match ){
+						if( g_pattern_match_simple( positive ? pattern : pattern+1, bname_utf8 )){
+							if( positive ){
+								match = TRUE;
+							} else {
+								ok = FALSE;
+							}
+						}
+					}
+
+					g_free( pattern );
+				}
+
+				ok &= match;
+
+				g_free( bname_utf8 );
+				g_free( bname );
+			}
+		}
 
 		if( !ok ){
 			gchar *basenames_str = na_core_utils_slist_to_text( basenames );
@@ -734,10 +791,28 @@ is_candidate_for_selection_count( const NAIContext *object, guint target, GList
 {
 	static const gchar *thisfn = "na_icontext_is_candidate_for_selection_count";
 	gboolean ok = TRUE;
+	gint limit;
+	guint count;
 	gchar *selection_count = na_object_get_selection_count( object );
 
 	if( selection_count && strlen( selection_count )){
+		limit = atoi( selection_count+1 );
+		count = g_list_length( files );
+		ok = FALSE;
 
+		switch( selection_count[0] ){
+			case '<':
+				ok = ( count < limit );
+				break;
+			case '=':
+				ok = ( count == limit );
+				break;
+			case '>':
+				ok = ( count > limit );
+				break;
+			default:
+				break;
+		}
 	}
 
 	if( !ok ){
@@ -749,6 +824,16 @@ is_candidate_for_selection_count( const NAIContext *object, guint target, GList
 	return( ok );
 }
 
+/*
+ * it is likely that all selected items have the same scheme, because they
+ * are all in the same location and the scheme mainly depends on location
+ * so we have here a great possible optimization by only testing the
+ * first selected item.
+ * note that this optimization may be wrong, for example when ran from the
+ * command-line with a random set of pseudo-selected items
+ * so we take care of only checking _distincts_ schemes of provided selection
+ * against schemes conditions.
+ */
 static gboolean
 is_candidate_for_schemes( const NAIContext *object, guint target, GList *files )
 {
@@ -757,6 +842,44 @@ is_candidate_for_schemes( const NAIContext *object, guint target, GList *files )
 	GSList *schemes = na_object_get_schemes( object );
 
 	if( schemes ){
+		if( strcmp( schemes->data, "*" ) != 0 || g_slist_length( schemes ) > 1 ){
+			GSList *distincts = NULL;
+			GList *it;
+
+			for( it = files ; it && ok ; it = it->next ){
+				gchar *scheme = na_selected_info_get_uri_scheme( NA_SELECTED_INFO( files->data ));
+
+				if( na_core_utils_slist_count( distincts, scheme ) == 0 ){
+					GSList *is;
+					gchar *pattern;
+					gboolean match, positive;
+
+					match = FALSE;
+					distincts = g_slist_prepend( distincts, g_strdup( scheme ));
+
+					for( is = schemes ; is && ok ; is = is->next ){
+						pattern = ( gchar * ) is->data;
+						positive = is_positive_assertion( pattern );
+
+						if( !positive || !match ){
+							if( is_compatible_scheme( positive ? pattern : pattern+1, scheme )){
+								if( positive ){
+									match = TRUE;
+								} else {
+									ok = FALSE;
+								}
+							}
+						}
+					}
+
+					ok &= match;
+				}
+
+				g_free( scheme );
+			}
+
+			na_core_utils_slist_free( distincts );
+		}
 
 		if( !ok ){
 			gchar *schemes_str = na_core_utils_slist_to_text( schemes );
@@ -770,6 +893,29 @@ is_candidate_for_schemes( const NAIContext *object, guint target, GList *files )
 	return( ok );
 }
 
+static gboolean
+is_compatible_scheme( const gchar *pattern, const gchar *scheme )
+{
+	gboolean compatible;
+
+	compatible = FALSE;
+
+	if( strcmp( pattern, "*" )){
+		compatible = TRUE;
+	} else {
+		compatible = ( strcmp( pattern, scheme ) == 0 );
+	}
+
+	return( compatible );
+}
+
+/*
+ * assumuing here the same sort of optimization than for schemes
+ * i.e. we assume that all selected items are must probably located
+ * in the same dirname
+ * so we take care of only checking _distinct_ dirnames against folder
+ * conditions
+ */
 static gboolean
 is_candidate_for_folders( const NAIContext *object, guint target, GList *files )
 {
@@ -778,6 +924,48 @@ is_candidate_for_folders( const NAIContext *object, guint target, GList *files )
 	GSList *folders = na_object_get_folders( object );
 
 	if( folders ){
+		if( strcmp( folders->data, "/" ) != 0 || g_slist_length( folders ) > 1 ){
+			GSList *distincts = NULL;
+			GList *it;
+
+			for( it = files ; it && ok ; it = it->next ){
+				gchar *dirname = na_selected_info_get_dirname( NA_SELECTED_INFO( files->data ));
+
+				if( na_core_utils_slist_count( distincts, dirname ) == 0 ){
+					GSList *id;
+					gchar *dirname_utf8;
+					const gchar *pattern;
+					gboolean match, positive;
+
+					distincts = g_slist_prepend( distincts, g_strdup( dirname ));
+					dirname_utf8 = g_filename_to_utf8( dirname, -1, NULL, NULL, NULL );
+					match = FALSE;
+
+					for( id = folders ; id && ok ; id = id->next ){
+						pattern = ( const gchar * ) id->data;
+						positive = is_positive_assertion( pattern );
+
+						if( !positive || !match ){
+							if( g_pattern_match_simple( positive ? pattern : pattern+1, dirname_utf8 )){
+								if( positive ){
+									match = TRUE;
+								} else {
+									ok = FALSE;
+								}
+							}
+						}
+					}
+
+					ok &= match;
+
+					g_free( dirname_utf8 );
+				}
+
+				g_free( dirname );
+			}
+
+			na_core_utils_slist_free( distincts );
+		}
 
 		if( !ok ){
 			gchar *folders_str = na_core_utils_slist_to_text( folders );
@@ -791,34 +979,6 @@ is_candidate_for_folders( const NAIContext *object, guint target, GList *files )
 	return( ok );
 }
 
-#if 0
-static gboolean
-is_current_folder_inside( const NAIContext *object, NASelectedInfo *current_folder )
-{
-	gboolean is_inside;
-	GSList *folders, *ifold;
-	const gchar *path;
-	gchar *current_folder_path;
-
-	is_inside = FALSE;
-	current_folder_path = na_selected_info_get_path( current_folder );
-	folders = na_object_get_folders( object );
-
-	for( ifold = folders ; ifold && !is_inside ; ifold = ifold->next ){
-		path = ( const gchar * ) ifold->data;
-		if( path && g_utf8_strlen( path, -1 )){
-			is_inside = g_str_has_prefix( current_folder_path, path );
-			g_debug( "na_object_object_is_current_folder_inside: current_folder_path=%s, path=%s, is_inside=%s", current_folder_path, path, is_inside ? "True":"False" );
-		}
-	}
-
-	na_core_utils_slist_free( folders );
-	g_free( current_folder_path );
-
-	return( is_inside );
-}
-#endif
-
 static gboolean
 is_candidate_for_capabilities( const NAIContext *object, guint target, GList *files )
 {
@@ -827,6 +987,39 @@ is_candidate_for_capabilities( const NAIContext *object, guint target, GList *fi
 	GSList *capabilities = na_object_get_capabilities( object );
 
 	if( capabilities ){
+		GSList *ic;
+		GList *it;
+		const gchar *cap;
+		gboolean match, positive;
+
+		for( it = files ; it && ok ; it = it->next ){
+			for( ic = capabilities ; ic && ok ; ic = ic->next ){
+				cap = ( const gchar * ) ic->data;
+				positive = is_positive_assertion( cap );
+				match = FALSE;
+
+				if( !strcmp( positive ? cap : cap+1, "Owner" )){
+					match = na_selected_info_is_owner( NA_SELECTED_INFO( it->data ), getlogin());
+
+				} else if( !strcmp( positive ? cap : cap+1, "Readable" )){
+					match = na_selected_info_is_readable( NA_SELECTED_INFO( it->data ));
+
+				} else if( !strcmp( positive ? cap : cap+1, "Writable" )){
+					match = na_selected_info_is_writable( NA_SELECTED_INFO( it->data ));
+
+				} else if( !strcmp( positive ? cap : cap+1, "Executable" )){
+					match = na_selected_info_is_executable( NA_SELECTED_INFO( it->data ));
+
+				} else if( !strcmp( positive ? cap : cap+1, "Local" )){
+					match = na_selected_info_is_local( NA_SELECTED_INFO( it->data ));
+
+				} else {
+					g_warning( "%s: unknown capability %s", thisfn, cap );
+				}
+
+				ok &= (( positive && match ) || ( !positive && !match ));
+			}
+		}
 
 		if( !ok ){
 			gchar *capabilities_str = na_core_utils_slist_to_text( capabilities );
@@ -840,230 +1033,6 @@ is_candidate_for_capabilities( const NAIContext *object, guint target, GList *fi
 	return( ok );
 }
 
-#if 0
-static gboolean
-is_target_selection_candidate( const NAIContext *object, GList *files )
-{
-	gboolean retv = FALSE;
-	GSList *basenames, *mimetypes, *schemes;
-	gboolean matchcase, multiple, isdir, isfile;
-	gboolean test_multiple_file = FALSE;
-	gboolean test_file_type = FALSE;
-	gboolean test_scheme = FALSE;
-	gboolean test_basename = FALSE;
-	gboolean test_mimetype = FALSE;
-	GList* glob_patterns = NULL;
-	GList* glob_mime_patterns = NULL;
-	GSList* iter;
-	GList* iter1;
-	GList* iter2;
-	guint dir_count = 0;
-	guint file_count = 0;
-	guint total_count = 0;
-	guint scheme_ok_count = 0;
-	guint glob_ok_count = 0;
-	guint mime_glob_ok_count = 0;
-	gboolean basename_match_ok = FALSE;
-	gboolean mimetype_match_ok = FALSE;
-	gchar *tmp_pattern, *tmp_filename, *tmp_filename2, *tmp_mimetype, *tmp_mimetype2;
-
-	basenames = na_object_get_basenames( object );
-	matchcase = na_object_get_matchcase( object );
-	multiple = na_object_is_multiple( object );
-	isdir = na_object_is_dir( object );
-	isfile = na_object_is_file( object );
-	mimetypes = na_object_get_mimetypes( object );
-	schemes = na_object_get_schemes( object );
-
-	if( basenames && basenames->next != NULL &&
-			g_ascii_strcasecmp(( gchar * )( basenames->data ), "*" ) == 0 ){
-		/* if the only pattern is '*' then all files will match, so it
-		 * is not necessary to make the test for each of them
-		 */
-		test_basename = TRUE;
-
-	} else {
-		for (iter = basenames ; iter ; iter = iter->next ){
-
-			tmp_pattern = ( gchar * ) iter->data;
-			if( !matchcase ){
-				/* --> if case-insensitive asked, lower all the string
-				 * since the pattern matching function don't manage it
-				 * itself.
-				 */
-				tmp_pattern = g_ascii_strdown(( gchar * ) iter->data, strlen(( gchar * ) iter->data ));
-			}
-
-			glob_patterns = g_list_append( glob_patterns, g_pattern_spec_new( tmp_pattern ));
-
-			if( !matchcase ){
-				g_free( tmp_pattern );
-			}
-		}
-	}
-
-	if( mimetypes && mimetypes->next != NULL &&
-			( g_ascii_strcasecmp(( gchar * )( mimetypes->data ), "*" ) == 0 ||
-			  g_ascii_strcasecmp(( gchar * )( mimetypes->data), "*/*") == 0 )){
-		/* if the only pattern is '*' or * / * then all mimetypes will
-		 * match, so it is not necessary to make the test for each of them
-		 */
-		test_mimetype = TRUE;
-
-	} else {
-		for( iter = mimetypes ; iter ; iter = iter->next ){
-			glob_mime_patterns = g_list_append( glob_mime_patterns, g_pattern_spec_new(( gchar * ) iter->data ));
-		}
-	}
-
-	for( iter1 = files ; iter1 ; iter1 = iter1->next ){
-
-		tmp_filename = na_selected_info_get_path( NA_SELECTED_INFO( iter1->data ));
-		g_debug( "na_icontext_is_target_selection_candidate: tmp_filename=%s", tmp_filename );
-
-		if( tmp_filename ){
-			tmp_mimetype = na_selected_info_get_mime_type( NA_SELECTED_INFO( iter1->data ));
-			g_debug( "na_icontext_is_target_selection_candidate: tmp_mimetype=%s", tmp_mimetype );
-
-			if( !matchcase ){
-				/* --> if case-insensitive asked, lower all the string
-				 * since the pattern matching function don't manage it
-				 * itself.
-				 */
-				tmp_filename2 = g_ascii_strdown( tmp_filename, strlen( tmp_filename ));
-				g_free( tmp_filename );
-				tmp_filename = tmp_filename2;
-				g_debug( "na_icontext_is_target_selection_candidate: tmp_filename=%s", tmp_filename );
-			}
-
-			/* --> for the moment we deal with all mimetypes case-insensitively
-			 * note that a symlink to a directory has a 'inode/directory' mimetype
-			 * and, in general, a symlink to a target has the target's mimetype
-			 */
-			tmp_mimetype2 = g_ascii_strdown( tmp_mimetype, strlen( tmp_mimetype ));
-			g_free( tmp_mimetype );
-			tmp_mimetype = tmp_mimetype2;
-
-			if( na_selected_info_is_directory( NA_SELECTED_INFO( iter1->data ))){
-				dir_count++;
-			} else {
-				file_count++;
-			}
-
-			scheme_ok_count += validate_schemes( schemes, NA_SELECTED_INFO( iter1->data ));
-
-			if( !test_basename ){ /* if it is already ok, skip the test to improve performance */
-				basename_match_ok = FALSE;
-				iter2 = glob_patterns;
-				while( iter2 && !basename_match_ok ){
-					if( g_pattern_match_string(( GPatternSpec * ) iter2->data, tmp_filename )){
-						basename_match_ok = TRUE;
-					}
-					iter2 = iter2->next;
-				}
-
-				if( basename_match_ok ){
-					glob_ok_count++;
-				}
-			}
-
-			if( !test_mimetype ){ /* if it is already ok, skip the test to improve performance */
-				mimetype_match_ok = FALSE;
-				iter2 = glob_mime_patterns;
-				while( iter2 && !mimetype_match_ok ){
-					if (g_pattern_match_string(( GPatternSpec * ) iter2->data, tmp_mimetype )){
-						mimetype_match_ok = TRUE;
-					}
-					iter2 = iter2->next;
-				}
-
-				if( mimetype_match_ok ){
-					mime_glob_ok_count++;
-				}
-			}
-
-			g_free( tmp_mimetype );
-			g_free( tmp_filename );
-		}
-
-		total_count++;
-	}
-
-	if(( files != NULL ) && ( files->next == NULL ) && ( !multiple )){
-		test_multiple_file = TRUE;
-
-	} else if( multiple ){
-		test_multiple_file = TRUE;
-	}
-
-	if( isdir && isfile ){
-		if( dir_count > 0 || file_count > 0 ){
-			test_file_type = TRUE;
-		}
-	} else if( isdir && !isfile ){
-		if( file_count == 0 ){
-			test_file_type = TRUE;
-		}
-	} else if( !isdir && isfile ){
-		if( dir_count == 0 ){
-			test_file_type = TRUE;
-		}
-	}
-
-	if( scheme_ok_count == total_count ){
-		test_scheme = TRUE;
-	}
-
-	if( !test_basename ){ /* if not already tested */
-		if( glob_ok_count == total_count ){
-			test_basename = TRUE;
-		}
-	}
-
-	if( !test_mimetype ){ /* if not already tested */
-		if( mime_glob_ok_count == total_count ){
-			test_mimetype = TRUE;
-		}
-	}
-
-	if( test_basename && test_mimetype && test_file_type && test_scheme && test_multiple_file ){
-		retv = TRUE;
-	}
-
-	g_list_foreach (glob_patterns, (GFunc) g_pattern_spec_free, NULL);
-	g_list_free (glob_patterns);
-	g_list_foreach (glob_mime_patterns, (GFunc) g_pattern_spec_free, NULL);
-	g_list_free (glob_mime_patterns);
-	na_core_utils_slist_free( schemes );
-	na_core_utils_slist_free( mimetypes );
-	na_core_utils_slist_free( basenames );
-
-	return retv;
-}
-
-static gboolean
-validate_schemes( GSList *object_schemes, NASelectedInfo *nfi )
-{
-	gboolean is_ok;
-	GSList* iter;
-	gchar *scheme;
-
-	is_ok = FALSE;
-
-	for( iter = object_schemes ; iter && !is_ok ; iter = iter->next ){
-		scheme = na_selected_info_get_uri_scheme( nfi );
-
-		if( g_ascii_strncasecmp( scheme, ( gchar * ) iter->data, strlen(( gchar * ) iter->data )) == 0 ){
-			is_ok = TRUE;
-		}
-
-		g_free( scheme );
-	}
-
-	return( is_ok );
-}
-#endif
-
 static gboolean
 is_valid_basenames( const NAIContext *object )
 {
