@@ -83,8 +83,13 @@ static gboolean     ifactory_object_is_valid( const NAIFactoryObject *object );
 static void         ifactory_object_read_done( NAIFactoryObject *instance, const NAIFactoryProvider *reader, void *reader_data, GSList **messages );
 static guint        ifactory_object_write_done( NAIFactoryObject *instance, const NAIFactoryProvider *writer, void *writer_data, GSList **messages );
 
-static void         icontext_conditions_iface_init( NAIContextInterface *iface );
+static void         icontext_iface_init( NAIContextInterface *iface );
+static gboolean     icontext_is_candidate( NAIContext *object, guint target, GList *selection );
 
+static gboolean     convert_pre_v3_parameters( NAObjectProfile *profile );
+static gboolean     convert_pre_v3_parameters_str( gchar *str );
+static gboolean     convert_pre_v3_multiple( NAObjectProfile *profile );
+static gboolean     convert_pre_v3_isfiledir( NAObjectProfile *profile );
 static gboolean     profile_is_valid( const NAObjectProfile *profile );
 static gboolean     is_valid_path_parameters( const NAObjectProfile *profile );
 
@@ -120,8 +125,8 @@ register_type( void )
 		( GInstanceInitFunc ) instance_init
 	};
 
-	static const GInterfaceInfo icontext_conditions_iface_info = {
-		( GInterfaceInitFunc ) icontext_conditions_iface_init,
+	static const GInterfaceInfo icontext_iface_info = {
+		( GInterfaceInitFunc ) icontext_iface_init,
 		NULL,
 		NULL
 	};
@@ -136,7 +141,7 @@ register_type( void )
 
 	type = g_type_register_static( NA_OBJECT_ID_TYPE, "NAObjectProfile", &info, 0 );
 
-	g_type_add_interface_static( type, NA_ICONTEXT_TYPE, &icontext_conditions_iface_info );
+	g_type_add_interface_static( type, NA_ICONTEXT_TYPE, &icontext_iface_info );
 
 	g_type_add_interface_static( type, NA_IFACTORY_OBJECT_TYPE, &ifactory_object_iface_info );
 
@@ -181,13 +186,13 @@ instance_init( GTypeInstance *instance, gpointer klass )
 	static const gchar *thisfn = "na_object_profile_instance_init";
 	NAObjectProfile *self;
 
-	g_debug( "%s: instance=%p (%s), klass=%p",
-			thisfn, ( void * ) instance, G_OBJECT_TYPE_NAME( instance ), ( void * ) klass );
-
 	g_return_if_fail( NA_IS_OBJECT_PROFILE( instance ));
 
 	self = NA_OBJECT_PROFILE( instance );
 
+	g_debug( "%s: instance=%p (%s), klass=%p",
+			thisfn, ( void * ) instance, G_OBJECT_TYPE_NAME( instance ), ( void * ) klass );
+
 	self->private = g_new0( NAObjectProfilePrivate, 1 );
 
 	self->private->dispose_has_run = FALSE;
@@ -223,14 +228,14 @@ instance_dispose( GObject *object )
 	static const gchar *thisfn = "na_object_profile_instance_dispose";
 	NAObjectProfile *self;
 
-	g_debug( "%s: object=%p (%s)", thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
-
 	g_return_if_fail( NA_IS_OBJECT_PROFILE( object ));
 
 	self = NA_OBJECT_PROFILE( object );
 
 	if( !self->private->dispose_has_run ){
 
+		g_debug( "%s: object=%p (%s)", thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
+
 		self->private->dispose_has_run = TRUE;
 
 		/* chain up to the parent class */
@@ -246,12 +251,12 @@ instance_finalize( GObject *object )
 	static const gchar *thisfn = "na_object_profile_instance_finalize";
 	NAObjectProfile *self;
 
-	g_debug( "%s: object=%p (%s)", thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
-
 	g_return_if_fail( NA_IS_OBJECT_PROFILE( object ));
 
 	self = NA_OBJECT_PROFILE( object );
 
+	g_debug( "%s: object=%p (%s)", thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
+
 	g_free( self->private );
 
 	/* chain call to parent class */
@@ -327,6 +332,34 @@ ifactory_object_is_valid( const NAIFactoryObject *object )
 static void
 ifactory_object_read_done( NAIFactoryObject *instance, const NAIFactoryProvider *reader, void *reader_data, GSList **messages )
 {
+	static const gchar *thisfn = "na_object_profile_ifactory_object_read_done";
+	NAObjectAction *action;
+	guint iversion;
+
+	g_debug( "%s: profile=%p", thisfn, ( void * ) instance );
+
+	/* converts pre-v3 data
+	 */
+	action = NA_OBJECT_ACTION( na_object_get_parent( instance ));
+	iversion = na_object_get_iversion( action );
+	g_debug( "%s: iversion=%d", thisfn, iversion );
+
+	if( iversion < 3 ){
+
+		if( convert_pre_v3_parameters( NA_OBJECT_PROFILE( instance )) ||
+			convert_pre_v3_multiple( NA_OBJECT_PROFILE( instance )) ||
+			convert_pre_v3_isfiledir( NA_OBJECT_PROFILE( instance ))){
+
+				na_object_set_iversion( action, 3 );
+		}
+	}
+
+	/* prepare the context after the reading
+	 */
+	na_icontext_read_done( NA_ICONTEXT( instance ));
+
+	/* last, set other action defaults
+	 */
 	na_factory_object_set_defaults( instance );
 }
 
@@ -337,11 +370,219 @@ ifactory_object_write_done( NAIFactoryObject *instance, const NAIFactoryProvider
 }
 
 static void
-icontext_conditions_iface_init( NAIContextInterface *iface )
+icontext_iface_init( NAIContextInterface *iface )
 {
-	static const gchar *thisfn = "na_object_profile_icontext_conditions_iface_init";
+	static const gchar *thisfn = "na_object_profile_icontext_iface_init";
 
 	g_debug( "%s: iface=%p", thisfn, ( void * ) iface );
+
+	iface->is_candidate = icontext_is_candidate;
+}
+
+static gboolean
+icontext_is_candidate( NAIContext *object, guint target, GList *selection )
+{
+	return( TRUE );
+}
+
+/*
+ * starting wih v3, parameters are relabeled
+ *   pre-v3 parameters					post-v3 parameters
+ *   ----------------------------		-----------------------------------
+ *   									%b: (first) basename	(was %f)
+ *   									%B: list of basenames	(was %m)
+ *   									%c: count				(new)
+ * 	 %d: (first) base directory			...................		(unchanged)
+ * 										%D: list of base dir	(new)
+ *   %f: (first) filename	-> %b		%f: (first) pathname	(new)
+ *   									%F: list of pathnames	(was %M)
+ *   %h: (first) hostname				...................		(unchanged)
+ *   %m: list of basenames	-> %B		-						(removed)
+ *   %M: list of pathnames	-> %F		-						(removed)
+ *   									%n: (first) username	(was %U)
+ *   %p: (first) port number			...................		(unchanged)
+ *   %R: list of URIs		-> %U		-						(removed)
+ *   %s: (first) scheme					...................		(unchanged)
+ *   %u: (first) URI					...................		(unchanged)
+ *   %U: (first) username	-> %n		%U: list of URIs		(was %R)
+ *   									%w: (first) basename w/o ext.	(new)
+ *   									%W: list of basenames w/o ext.	(new)
+ *   									%x: (first) extension	(new)
+ *   									%X: list of extensions	(new)
+ *   %%: %								...................		(unchanged)
+ *
+ * For pre-v3 items,
+ * - substitute %f with %b
+ * - substitute %m with %B
+ * - substitute %M with %F
+ * - substitute %U with %n
+ * - substitute %R with %U
+ *
+ * Note that pre-v3 items only have parameters in the command and path fields.
+ * Are only located in 'profile' objects.
+ * Are only found in GConf or XML providers, as .desktop files have been
+ * simultaneously introduced.
+ *
+ * As a recall of the dynamics of the reading when loading an action:
+ *  - na_object_action_read_done: set action defaults
+ *  - nagp_reader_read_done: read profiles
+ *     > nagp_reader_read_start: attach profile to its parent
+ *     >  na_object_profile_read_done: convert old parameters
+ *
+ * So, when converting v2 to v3 parameters in a v2 profile,
+ * action already has its default values (including iversion=3)
+ */
+static gboolean
+convert_pre_v3_parameters( NAObjectProfile *profile )
+{
+	gboolean path_changed, parms_changed;
+
+	gchar *path = na_object_get_path( profile );
+	path_changed = convert_pre_v3_parameters_str( path );
+	if( path_changed ){
+		na_object_set_path( profile, path );
+	}
+	g_free( path );
+
+	gchar *parms = na_object_get_parameters( profile );
+	parms_changed = convert_pre_v3_parameters_str( parms );
+	if( parms_changed ){
+		na_object_set_parameters( profile, parms );
+	}
+	g_free( parms );
+
+	return( path_changed || parms_changed );
+}
+
+static gboolean
+convert_pre_v3_parameters_str( gchar *str )
+{
+	gboolean changed;
+	gchar *iter = str;
+
+	changed = FALSE;
+	while( iter != NULL &&
+			strlen( iter ) > 0 &&
+			( iter = g_strstr_len( iter, strlen( iter ), "%" )) != NULL ){
+
+		g_debug( "convert_pre_v3_parameters_str: iter[1]='%c'", iter[1] );
+		switch( iter[1] ){
+
+			/* %f (first filename) becomes %b
+			 */
+			case 'f':
+				iter[1] = 'b';
+				changed = TRUE;
+				break;
+
+			/* %m (list of basenames) becomes %B
+			 */
+			case 'm':
+				iter[1] = 'B';
+				changed = TRUE;
+				break;
+
+			/* %M (list of filenames) becomes %F
+			 */
+			case 'M':
+				iter[1] = 'F';
+				changed = TRUE;
+				break;
+
+			/* %U ((first) username) becomes %n
+			 */
+			case 'U':
+				iter[1] = 'n';
+				changed = TRUE;
+				break;
+
+			/* %R (list of URIs) becomes %U
+			 */
+			case 'R':
+				iter[1] = 'U';
+				changed = TRUE;
+				break;
+		}
+
+		iter += 2;
+	}
+
+	return( changed );
+}
+
+/*
+ * default changes from accept_multiple=false
+ *                   to selection_count>0
+ */
+static gboolean
+convert_pre_v3_multiple( NAObjectProfile *profile )
+{
+	gboolean accept_multiple;
+	gchar *selection_count;
+
+	accept_multiple = na_object_is_multiple( profile );
+	selection_count = g_strdup( accept_multiple ? ">0" : "=1" );
+	na_object_set_selection_count( profile, selection_count );
+	g_free( selection_count );
+
+	return( TRUE );
+}
+
+/*
+ * we may have file=true  and dir=false -> only files          -> all/allfiles
+ *             file=false and dir=true  -> only dirs           -> inode/directory
+ *             file=true  and dir=true  -> both files and dirs -> all/all
+ *
+ * we try to replace this with the corresponding mimetype, but only if
+ * current mimetype is '*' (or * / * or all/all)
+ */
+static gboolean
+convert_pre_v3_isfiledir( NAObjectProfile *profile )
+{
+	gboolean converted;
+	gboolean isfile, isdir;
+	GSList *mimetypes;
+
+	converted = FALSE;
+
+	if( na_icontext_is_all_mimetypes( NA_ICONTEXT( profile ))){
+		converted = TRUE;
+		mimetypes = NULL;
+
+		isfile = na_object_is_file( profile );
+		isdir = na_object_is_dir( profile );
+
+		if( isfile ){
+			if( isdir ){
+				/* both file and dir -> do not modify mimetypes
+				 */
+				converted = FALSE;
+			} else {
+				/* files only
+				 */
+				mimetypes = g_slist_prepend( NULL, g_strdup( "all/allfiles" ));
+			}
+		} else {
+			if( isdir ){
+				/* dir only
+				 */
+				mimetypes = g_slist_prepend( NULL, g_strdup( "inode/directory" ));
+			} else {
+				/* not files nor dir: this is an invalid case -> do not modify
+				 * mimetypes
+				 */
+				converted = FALSE;
+			}
+		}
+
+		if( converted ){
+			na_object_set_mimetypes( profile, mimetypes );
+		}
+
+		na_core_utils_slist_free( mimetypes );
+	}
+
+	return( converted );
 }
 
 static gboolean
@@ -403,11 +644,13 @@ object_id_new_id( const NAObjectId *item, const NAObjectId *new_parent )
 	gchar *id = NULL;
 
 	g_return_val_if_fail( NA_IS_OBJECT_PROFILE( item ), NULL );
-	g_return_val_if_fail( new_parent && NA_IS_OBJECT_ACTION( new_parent ), NULL );
+	g_return_val_if_fail( !new_parent || NA_IS_OBJECT_ACTION( new_parent ), NULL );
 
 	if( !NA_OBJECT_PROFILE( item )->private->dispose_has_run ){
 
-		id = na_object_action_get_new_profile_name( NA_OBJECT_ACTION( new_parent ));
+		if( new_parent ){
+			id = na_object_action_get_new_profile_name( NA_OBJECT_ACTION( new_parent ));
+		}
 	}
 
 	return( id );
@@ -448,226 +691,3 @@ na_object_profile_new_with_defaults( void )
 
 	return( profile );
 }
-
-/**
- * Expands the parameters path, in function of the found tokens.
- *
- * @profile: the selected profile.
- * @target: the current target.
- * @files: the list of currently selected #NASelectedInfo items.
- *
- * Valid parameters are :
- *
- * %d : base dir of the (first) selected file(s)/folder(s)
- * %f : the name of the (first) selected file/folder
- * %h : hostname of the (first) URI
- * %m : list of the basename of the selected files/directories separated by space.
- * %M : list of the selected files/directories with their complete path separated by space.
- * %p : port number from the (first) URI
- * %R : space-separated list of URIs
- * %s : scheme of the (first) URI
- * %u : (first) URI
- * %U : username of the (first) URI
- * %% : a percent sign
- *
- * Adding a parameter requires updating of :
- * - nautilus-actions/core/na-object-profile.c::na_object_profile_parse_parameters()
- * - nautilus-actions/nact/nact-icommand-tab.c:parse_parameters()
- * - nautilus-actions/nact/nautilus-actions-config-tool.ui:LegendDialog
- */
-gchar *
-na_object_profile_parse_parameters( const NAObjectProfile *profile, gint target, GList* files )
-{
-	gchar *parsed = NULL;
-	GString *string;
-	GList *ifi;
-	gboolean first;
-	gchar *iuri, *ipath, *ibname;
-	GFile *iloc;
-	gchar *uri = NULL;
-	gchar *scheme = NULL;
-	gchar *dirname = NULL;
-	gchar *filename = NULL;
-	gchar *hostname = NULL;
-	gchar *username = NULL;
-	gint port_number = 0;
-	GString *basename_list, *pathname_list, *uris_list;
-	gchar *tmp, *iter, *old_iter;
-	NAGnomeVFSURI *vfs;
-
-	g_return_val_if_fail( NA_IS_OBJECT_PROFILE( profile ), NULL );
-
-	if( profile->private->dispose_has_run ){
-		return( NULL );
-	}
-
-	string = g_string_new( "" );
-	basename_list = g_string_new( "" );
-	pathname_list = g_string_new( "" );
-	uris_list = g_string_new( "" );
-	first = TRUE;
-
-	for( ifi = files ; ifi ; ifi = ifi->next ){
-
-		iuri = na_selected_info_get_uri( NA_SELECTED_INFO( ifi->data ));
-		iloc = na_selected_info_get_location( NA_SELECTED_INFO( ifi->data ));
-		ipath = g_file_get_path( iloc );
-		ibname = g_file_get_basename( iloc );
-
-		if( first ){
-
-			vfs = g_new0( NAGnomeVFSURI, 1 );
-			na_gnome_vfs_uri_parse( vfs, iuri );
-
-			uri = g_strdup( iuri );
-			dirname = ipath ? g_path_get_dirname( ipath ) : NULL;
-			scheme = g_strdup( vfs->scheme );
-			filename = g_strdup( ibname );
-			hostname = g_strdup( vfs->host_name );
-			username = g_strdup( vfs->user_name );
-			port_number = vfs->host_port;
-
-			first = FALSE;
-			na_gnome_vfs_uri_free( vfs );
-		}
-
-		if( ibname ){
-			tmp = g_shell_quote( ibname );
-			g_string_append_printf( basename_list, " %s", tmp );
-			g_free( tmp );
-		}
-
-		if( ipath ){
-			tmp = g_shell_quote( ipath );
-			g_string_append_printf( pathname_list, " %s", tmp );
-			g_free( tmp );
-		}
-
-		tmp = g_shell_quote( iuri );
-		g_string_append_printf( uris_list, " %s", tmp );
-		g_free( tmp );
-
-		g_free( ibname );
-		g_free( ipath );
-		g_object_unref( iloc );
-		g_free( iuri );
-	}
-
-	iter = na_object_get_parameters( profile );
-	old_iter = iter;
-
-	while(( iter = g_strstr_len( iter, strlen( iter ), "%" ))){
-
-		string = g_string_append_len( string, old_iter, strlen( old_iter ) - strlen( iter ));
-		switch( iter[1] ){
-
-			/* base dir of the (first) selected item
-			 */
-			case 'd':
-				if( dirname ){
-					tmp = g_shell_quote( dirname );
-					string = g_string_append( string, tmp );
-					g_free( tmp );
-				}
-				break;
-
-			/* basename of the (first) selected item
-			 */
-			case 'f':
-				if( filename ){
-					tmp = g_shell_quote( filename );
-					string = g_string_append( string, tmp );
-					g_free( tmp );
-				}
-				break;
-
-			/* hostname of the (first) URI
-			 */
-			case 'h':
-				if( hostname ){
-					string = g_string_append( string, hostname );
-				}
-				break;
-
-			/* space-separated list of the basenames
-			 */
-			case 'm':
-				if( basename_list->str ){
-					string = g_string_append( string, basename_list->str );
-				}
-				break;
-
-			/* space-separated list of full pathnames
-			 */
-			case 'M':
-				if( pathname_list->str ){
-					string = g_string_append( string, pathname_list->str );
-				}
-				break;
-
-			/* port number of the (first) URI
-			 */
-			case 'p':
-				if( port_number > 0 ){
-					g_string_append_printf( string, "%d", port_number );
-				}
-				break;
-
-			/* list of URIs
-			 */
-			case 'R':
-				if( uris_list->str ){
-					string = g_string_append( string, uris_list->str );
-				}
-				break;
-
-			/* scheme of the (first) URI
-			 */
-			case 's':
-				if( scheme ){
-					string = g_string_append( string, scheme );
-				}
-				break;
-
-			/* URI of the first item
-			 */
-			case 'u':
-				if( uri ){
-					string = g_string_append( string, uri );
-				}
-				break;
-
-			/* username of the (first) URI
-			 */
-			case 'U':
-				if( username ){
-					string = g_string_append( string, username );
-				}
-				break;
-
-			/* a percent sign
-			 */
-			case '%':
-				string = g_string_append_c( string, '%' );
-				break;
-		}
-
-		iter += 2;			/* skip the % sign and the character after */
-		old_iter = iter;	/* store the new start of the string */
-	}
-
-	string = g_string_append_len( string, old_iter, strlen( old_iter ));
-
-	g_free( uri );
-	g_free( dirname );
-	g_free( scheme );
-	g_free( hostname );
-	g_free( username );
-	g_free( iter );
-	g_string_free( uris_list, TRUE );
-	g_string_free( basename_list, TRUE );
-	g_string_free( pathname_list, TRUE );
-
-	parsed = g_string_free( string, FALSE );
-	return( parsed );
-}
