@@ -4,7 +4,7 @@
  *
  * Copyright (C) 2005 The GNOME Foundation
  * Copyright (C) 2006, 2007, 2008 Frederic Ruaudel and others (see AUTHORS)
- * Copyright (C) 2009 Pierre Wieser and others (see AUTHORS)
+ * Copyright (C) 2009, 2010 Pierre Wieser and others (see AUTHORS)
  *
  * This Program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -35,16 +35,14 @@
 #include <gtk/gtk.h>
 #include <string.h>
 
-#include <runtime/na-pivot.h>
+#include <api/na-object-api.h>
 
-#include <common/na-iprefs.h>
-#include <common/na-object-api.h>
-#include <common/na-xml-names.h>
-#include <common/na-xml-writer.h>
-#include <common/na-utils.h>
+#include <core/na-exporter.h>
+#include <core/na-export-format.h>
 
 #include "nact-application.h"
-#include "nact-assistant-export-ask.h"
+#include "nact-iprefs.h"
+#include "nact-export-ask.h"
 #include "nact-tree-model.h"
 #include "nact-clipboard.h"
 
@@ -56,11 +54,29 @@ struct NactClipboardClassPrivate {
 
 /* private instance data
  */
+typedef struct {
+	guint    target;
+	gchar   *folder;
+	GList   *rows;
+	gboolean copy;
+}
+	NactClipboardDndData;
+
+typedef struct {
+	GList *items;
+	gint   mode;
+	guint  nb_actions;
+	guint  nb_profiles;
+	guint  nb_menus;
+}
+	PrimaryData;
+
 struct NactClipboardPrivate {
 	gboolean      dispose_has_run;
 	BaseWindow   *window;
 	GtkClipboard *dnd;
 	GtkClipboard *primary;
+	PrimaryData  *primary_data;
 	gboolean      primary_got;
 };
 
@@ -87,23 +103,6 @@ static GtkTargetEntry clipboard_formats[] = {
 	{ "text/plain",               0, NACT_CLIPBOARD_FORMAT_TEXT_PLAIN },
 };
 
-typedef struct {
-	guint    target;
-	gchar   *folder;
-	GList   *rows;
-	gboolean copy;
-}
-	NactClipboardDndData;
-
-typedef struct {
-	guint    nb_actions;
-	guint    nb_profiles;
-	guint    nb_menus;
-	GList   *items;
-	gint     mode;
-}
-	NactClipboardPrimaryData;
-
 static GObjectClass *st_parent_class = NULL;
 
 static GType  register_type( void );
@@ -115,10 +114,15 @@ static void   instance_finalize( GObject *application );
 static void   get_from_dnd_clipboard_callback( GtkClipboard *clipboard, GtkSelectionData *selection_data, guint info, guchar *data );
 static void   clear_dnd_clipboard_callback( GtkClipboard *clipboard, NactClipboardDndData *data );
 static gchar *export_rows( NactClipboard *clipboard, GList *rows, const gchar *dest_folder );
+static gchar *export_objects( NactClipboard *clipboard, GList *objects );
 static gchar *export_row_object( NactClipboard *clipboard, NAObject *object, const gchar *dest_folder, GList **exported );
 
-static void   get_from_primary_clipboard_callback( GtkClipboard *clipboard, GtkSelectionData *selection_data, guint info, guchar *data );
-static void   clear_primary_clipboard_callback( GtkClipboard *clipboard, NactClipboardPrimaryData *data );
+static void   get_from_primary_clipboard_callback( GtkClipboard *gtk_clipboard, GtkSelectionData *selection_data, guint info, NactClipboard *clipboard );
+static void   clear_primary_clipboard( NactClipboard *clipboard );
+static void   clear_primary_clipboard_callback( GtkClipboard *gtk_clipboard, NactClipboard *clipboard );
+static void   dump_primary_clipboard( NactClipboard *clipboard );
+
+static gchar *clipboard_mode_to_string( gint mode );
 
 GType
 nact_clipboard_get_type( void )
@@ -181,7 +185,8 @@ instance_init( GTypeInstance *instance, gpointer klass )
 	NactClipboard *self;
 	GdkDisplay *display;
 
-	g_debug( "%s: instance=%p, klass=%p", thisfn, ( void * ) instance, ( void * ) klass );
+	g_debug( "%s: instance=%p (%s), klass=%p",
+			thisfn, ( void * ) instance, G_OBJECT_TYPE_NAME( instance ), ( void * ) klass );
 	g_assert( NACT_IS_CLIPBOARD( instance ));
 	self = NACT_CLIPBOARD( instance );
 
@@ -191,18 +196,19 @@ instance_init( GTypeInstance *instance, gpointer klass )
 
 	display = gdk_display_get_default();
 	self->private->dnd = gtk_clipboard_get_for_display( display, NACT_CLIPBOARD_ATOM );
-	self->private->primary = gtk_clipboard_get_for_display( display, GDK_SELECTION_PRIMARY );
+	self->private->primary = gtk_clipboard_get_for_display( display, GDK_SELECTION_CLIPBOARD );
+	self->private->primary_data = NULL;
 }
 
 static void
-instance_dispose( GObject *window )
+instance_dispose( GObject *object )
 {
 	static const gchar *thisfn = "nact_clipboard_instance_dispose";
 	NactClipboard *self;
 
-	g_debug( "%s: window=%p", thisfn, ( void * ) window );
-	g_assert( NACT_IS_CLIPBOARD( window ));
-	self = NACT_CLIPBOARD( window );
+	g_debug( "%s: object=%p (%s)", thisfn, ( void * ) object, G_OBJECT_TYPE_NAME( object ));
+	g_assert( NACT_IS_CLIPBOARD( object ));
+	self = NACT_CLIPBOARD( object );
 
 	if( !self->private->dispose_has_run ){
 
@@ -213,7 +219,7 @@ instance_dispose( GObject *window )
 
 		/* chain up to the parent class */
 		if( G_OBJECT_CLASS( st_parent_class )->dispose ){
-			G_OBJECT_CLASS( st_parent_class )->dispose( window );
+			G_OBJECT_CLASS( st_parent_class )->dispose( object );
 		}
 	}
 }
@@ -228,6 +234,11 @@ instance_finalize( GObject *window )
 	g_assert( NACT_IS_CLIPBOARD( window ));
 	self = NACT_CLIPBOARD( window );
 
+	if( self->private->primary_data ){
+		clear_primary_clipboard( self );
+		g_free( self->private->primary_data );
+	}
+
 	g_free( self->private );
 
 	/* chain call to parent class */
@@ -259,7 +270,7 @@ nact_clipboard_new( BaseWindow *window )
  * nact_clipboard_dnd_set:
  * @clipboard: this #NactClipboard instance.
  * @rows: the list of row references of dragged items.
- * @folder: the target folder if any (XDS protocol to outside).
+ * @folder: the URI of the target folder if any (XDS protocol to outside).
  * @copy_data: %TRUE if data is to be copied, %FALSE else
  *  (only relevant when drag and drop occurs inside of the tree view).
  *
@@ -347,6 +358,7 @@ nact_clipboard_dnd_get_data( NactClipboard *clipboard, gboolean *copy_data )
 				*copy_data = data->copy;
 			}
 		}
+		gtk_selection_data_free( selection );
 	}
 
 	return( rows );
@@ -370,14 +382,20 @@ nact_clipboard_dnd_get_data( NactClipboard *clipboard, gboolean *copy_data )
 gchar *
 nact_clipboard_dnd_get_text( NactClipboard *clipboard, GList *rows )
 {
+	static const gchar *thisfn = "nact_clipboard_dnd_get_text";
 	gchar *buffer;
 
 	g_return_val_if_fail( NACT_IS_CLIPBOARD( clipboard ), NULL );
 
+	g_debug( "%s: clipboard=%p, rows=%p (count=%u)",
+			thisfn, ( void * ) clipboard, ( void * ) rows, g_list_length( rows ));
+
 	buffer = NULL;
 
 	if( !clipboard->private->dispose_has_run ){
+
 		buffer = export_rows( clipboard, rows, NULL );
+		g_debug( "%s: returning buffer=%p (length=%lu)", thisfn, ( void * ) buffer, g_utf8_strlen( buffer, -1 ));
 	}
 
 	return( buffer );
@@ -402,12 +420,18 @@ nact_clipboard_dnd_drag_end( NactClipboard *clipboard )
 	if( !clipboard->private->dispose_has_run ){
 
 		selection = gtk_clipboard_wait_for_contents( clipboard->private->dnd, NACT_CLIPBOARD_NACT_ATOM );
-		if( selection ){
+		g_debug( "%s: selection=%p", thisfn, ( void * ) selection );
 
+		if( selection ){
 			data = ( NactClipboardDndData * ) selection->data;
+			g_debug( "%s: data=%p (NactClipboardDndData)", thisfn, ( void * ) data );
+
 			if( data->target == NACT_XCHANGE_FORMAT_XDS ){
+				g_debug( "%s: folder=%s", thisfn, data->folder );
 				export_rows( clipboard, data->rows, data->folder );
 			}
+
+			gtk_selection_data_free( selection );
 		}
 	}
 }
@@ -423,6 +447,8 @@ nact_clipboard_dnd_drag_end( NactClipboard *clipboard )
 void
 nact_clipboard_dnd_clear( NactClipboard *clipboard )
 {
+	g_debug( "nact_clipboard_dnd_clear: clipboard=%p", ( void * ) clipboard );
+
 	gtk_clipboard_clear( clipboard->private->dnd );
 }
 
@@ -486,22 +512,55 @@ export_rows( NactClipboard *clipboard, GList *rows, const gchar *dest_folder )
 	return( g_string_free( data, FALSE ));
 }
 
+static gchar *
+export_objects( NactClipboard *clipboard, GList *objects )
+{
+	gchar *buffer;
+	GString *data;
+	GList *exported;
+	GList *iobj;
+	NAObject *object;
+
+	buffer = NULL;
+	exported = NULL;
+	data = g_string_new( "" );
+
+	for( iobj = objects ; iobj ; iobj = iobj->next ){
+		object = NA_OBJECT( iobj->data );
+		buffer = export_row_object( clipboard, object, NULL, &exported );
+		if( buffer && strlen( buffer )){
+			data = g_string_append( data, buffer );
+			g_free( buffer );
+		}
+		g_object_unref( object );
+	}
+
+	g_list_free( exported );
+	return( g_string_free( data, FALSE ));
+}
+
+/*
+ * export to a buffer if dest_folder is null
+ * else export to a new file in the target directory
+ */
 static gchar *
 export_row_object( NactClipboard *clipboard, NAObject *object, const gchar *dest_folder, GList **exported )
 {
 	GList *subitems, *isub;
+	NactApplication *application;
+	NAUpdater *updater;
 	NAObjectAction *action;
 	gint index;
 	GString *data;
 	gchar *buffer;
-	NactApplication *application;
-	NAPivot *pivot;
-	gint format;
+	GQuark format;
 	gchar *fname;
+	GSList *msgs;
+
+	data = g_string_new( "" );
 
 	if( NA_IS_OBJECT_MENU( object )){
-		subitems = na_object_get_items_list( object );
-		data = g_string_new( "" );
+		subitems = na_object_get_items( object );
 
 		for( isub = subitems ; isub ; isub = isub->next ){
 			buffer = export_row_object( clipboard, isub->data, dest_folder, exported );
@@ -510,42 +569,44 @@ export_row_object( NactClipboard *clipboard, NAObject *object, const gchar *dest
 				g_free( buffer );
 			}
 		}
-
-		return( g_string_free( data, FALSE ));
 	}
 
-	buffer = NULL;
+	msgs = NULL;
 	action = ( NAObjectAction * ) object;
 	if( NA_IS_OBJECT_PROFILE( object )){
 		action = NA_OBJECT_ACTION( na_object_get_parent( object ));
 	}
 
+	application = NACT_APPLICATION( base_window_get_application( clipboard->private->window ));
+	updater = nact_application_get_updater( application );
+
 	index = g_list_index( *exported, ( gconstpointer ) action );
 	if( index == -1 ){
-		*exported = g_list_prepend( *exported, ( gpointer ) action );
 
-		application = NACT_APPLICATION( base_window_get_application( clipboard->private->window ));
-		pivot = nact_application_get_pivot( application );
-		format = na_iprefs_get_export_format( NA_IPREFS( pivot ), IPREFS_EXPORT_FORMAT );
+		*exported = g_list_prepend( *exported, ( gpointer ) action );
+		format = nact_iprefs_get_export_format( clipboard->private->window, IPREFS_EXPORT_FORMAT );
 
 		if( format == IPREFS_EXPORT_FORMAT_ASK ){
-			format = nact_assistant_export_ask_user( clipboard->private->window, action );
+			format = nact_export_ask_user( clipboard->private->window, NA_OBJECT_ITEM( action ));
 		}
 
 		if( format != IPREFS_EXPORT_NO_EXPORT ){
-			buffer = na_xml_writer_get_xml_buffer( action, format );
 
-			if( buffer && dest_folder ){
-				fname = na_xml_writer_get_output_fname( action, dest_folder, format );
-				na_xml_writer_output_xml( buffer, fname );
+			if( dest_folder ){
+				fname = na_exporter_to_file( NA_PIVOT( updater ), NA_OBJECT_ITEM( action), dest_folder, format, &msgs );
 				g_free( fname );
-				g_free( buffer );
-				buffer = NULL;
+
+			} else {
+				buffer = na_exporter_to_buffer( NA_PIVOT( updater ), NA_OBJECT_ITEM( action ), format, &msgs );
+				if( buffer && strlen( buffer )){
+					data = g_string_append( data, buffer );
+					g_free( buffer );
+				}
 			}
 		}
 	}
 
-	return( buffer );
+	return( g_string_free( data, FALSE ));
 }
 
 /**
@@ -559,7 +620,7 @@ export_row_object( NactClipboard *clipboard, NAObject *object, const gchar *dest
  *
  * Rationale: when cutting an item to the clipboard, the next paste
  * will keep its same original id, and it is safe because this is
- * actually what we we want when we cut/paste.
+ * actually what we want when we cut/paste.
  *
  * Contrarily, when we copy/paste, we are waiting for a new element
  * which has the same characteristics that the previous one ; we so
@@ -572,32 +633,48 @@ export_row_object( NactClipboard *clipboard, NAObject *object, const gchar *dest
 void
 nact_clipboard_primary_set( NactClipboard *clipboard, GList *items, gint mode )
 {
-	NactClipboardPrimaryData *data;
+	static const gchar *thisfn = "nact_clipboard_primary_set";
+	PrimaryData *user_data;
 	GList *it;
 
+	g_debug( "%s: clipboard=%p, items=%p (count=%d), mode=%d",
+			thisfn, ( void * ) clipboard, ( void * ) items, g_list_length( items ), mode );
 	g_return_if_fail( NACT_IS_CLIPBOARD( clipboard ));
 
 	if( !clipboard->private->dispose_has_run ){
 
-		data = g_new0( NactClipboardPrimaryData, 1 );
+		user_data = clipboard->private->primary_data;
 
-		na_object_item_count_items( items,
-				( gint * ) &data->nb_menus, ( gint * ) &data->nb_actions, ( gint * ) &data->nb_profiles, FALSE );
+		if( user_data == NULL ){
+			user_data = g_new0( PrimaryData, 1 );
+			clipboard->private->primary_data = user_data;
+			g_debug( "%s: allocating PrimaryData=%p", thisfn, ( void * ) user_data );
+
+		} else {
+			clear_primary_clipboard( clipboard );
+		}
+
+		na_object_count_items( items,
+				( gint * ) &user_data->nb_menus,
+				( gint * ) &user_data->nb_actions,
+				( gint * ) &user_data->nb_profiles,
+				FALSE );
 
 		for( it = items ; it ; it = it->next ){
-			data->items = g_list_prepend( data->items, na_object_duplicate( it->data ));
+			user_data->items =
+					g_list_prepend( user_data->items, na_object_duplicate( it->data ));
 		}
-		data->items = g_list_reverse( data->items );
+		user_data->items = g_list_reverse( user_data->items );
 
-		data->mode = mode;
-
-		clipboard->private->primary_got = FALSE;
+		user_data->mode = mode;
 
 		gtk_clipboard_set_with_data( clipboard->private->primary,
 				clipboard_formats, G_N_ELEMENTS( clipboard_formats ),
 				( GtkClipboardGetFunc ) get_from_primary_clipboard_callback,
 				( GtkClipboardClearFunc ) clear_primary_clipboard_callback,
-				data );
+				clipboard );
+
+		clipboard->private->primary_got = FALSE;
 	}
 }
 
@@ -614,12 +691,14 @@ nact_clipboard_primary_set( NactClipboard *clipboard, GList *items, gint mode )
 GList *
 nact_clipboard_primary_get( NactClipboard *clipboard, gboolean *relabel )
 {
-	GtkSelectionData *selection;
-	NactClipboardPrimaryData *data;
+	static const gchar *thisfn = "nact_clipboard_primary_get";
 	GList *items = NULL;
+	GtkSelectionData *selection;
+	PrimaryData *user_data;
 	GList *it;
 	NAObject *obj;
 
+	g_debug( "%s: clipboard=%p", thisfn, ( void * ) clipboard );
 	g_return_val_if_fail( NACT_IS_CLIPBOARD( clipboard ), NULL );
 	g_return_val_if_fail( relabel, NULL );
 
@@ -628,19 +707,24 @@ nact_clipboard_primary_get( NactClipboard *clipboard, gboolean *relabel )
 		selection = gtk_clipboard_wait_for_contents( clipboard->private->primary, NACT_CLIPBOARD_NACT_ATOM );
 
 		if( selection ){
-			data = ( NactClipboardPrimaryData * ) selection->data;
+			user_data = ( PrimaryData * ) selection->data;
+			g_debug( "%s: retrieving PrimaryData=%p", thisfn, ( void * ) user_data );
 
-			for( it = data->items ; it ; it = it->next ){
-				obj = na_object_duplicate( it->data );
-				na_object_set_origin( obj, NULL );
-				items = g_list_prepend( items, obj );
+			if( user_data ){
+				for( it = user_data->items ; it ; it = it->next ){
+					obj = NA_OBJECT( na_object_duplicate( it->data ));
+					na_object_set_origin( obj, NULL );
+					items = g_list_prepend( items, obj );
+				}
+				items = g_list_reverse( items );
+
+				*relabel = (( user_data->mode == CLIPBOARD_MODE_CUT && clipboard->private->primary_got ) ||
+								user_data->mode == CLIPBOARD_MODE_COPY );
+
+				clipboard->private->primary_got = TRUE;
 			}
-			items = g_list_reverse( items );
 
-			*relabel = (( data->mode == CLIPBOARD_MODE_CUT && clipboard->private->primary_got ) ||
-							data->mode == CLIPBOARD_MODE_COPY );
-
-			clipboard->private->primary_got = TRUE;
+			gtk_selection_data_free( selection );
 		}
 	}
 
@@ -656,8 +740,7 @@ nact_clipboard_primary_get( NactClipboard *clipboard, gboolean *relabel )
 void
 nact_clipboard_primary_counts( NactClipboard *clipboard, guint *actions, guint *profiles, guint *menus )
 {
-	GtkSelectionData *selection;
-	NactClipboardPrimaryData *data;
+	PrimaryData *user_data;
 
 	g_return_if_fail( NACT_IS_CLIPBOARD( clipboard ));
 	g_return_if_fail( actions && profiles && menus );
@@ -668,38 +751,143 @@ nact_clipboard_primary_counts( NactClipboard *clipboard, guint *actions, guint *
 		*profiles = 0;
 		*menus = 0;
 
-		selection = gtk_clipboard_wait_for_contents( clipboard->private->primary, NACT_CLIPBOARD_NACT_ATOM );
+		user_data = clipboard->private->primary_data;
+		if( user_data ){
 
-		if( selection ){
-			data = ( NactClipboardPrimaryData * ) selection->data;
-
-			*actions = data->nb_actions;
-			*profiles = data->nb_profiles;
-			*menus = data->nb_menus;
+			*actions = user_data->nb_actions;
+			*profiles = user_data->nb_profiles;
+			*menus = user_data->nb_menus;
 		}
 	}
 }
 
 static void
-get_from_primary_clipboard_callback( GtkClipboard *clipboard, GtkSelectionData *selection_data, guint info, guchar *data )
+get_from_primary_clipboard_callback( GtkClipboard *gtk_clipboard, GtkSelectionData *selection_data, guint info, NactClipboard *clipboard )
 {
 	static const gchar *thisfn = "nact_clipboard_get_from_primary_clipboard_callback";
+	PrimaryData *user_data;
+	gchar *buffer;
 
-	g_debug( "%s: clipboard=%p, selection_data=%p, target=%s, info=%d, data=%p",
-			thisfn, ( void * ) clipboard,
-			( void * ) selection_data, gdk_atom_name( selection_data->target ), info, ( void * ) data );
+	g_debug( "%s: gtk_clipboard=%p, selection_data=%p, target=%s, info=%d, clipboard=%p",
+			thisfn, ( void * ) gtk_clipboard,
+			( void * ) selection_data, gdk_atom_name( selection_data->target ), info, ( void * ) clipboard );
 
-	gtk_selection_data_set( selection_data, selection_data->target, 8, data, sizeof( NactClipboardPrimaryData ));
+	user_data = clipboard->private->primary_data;
+
+	if( info == NACT_CLIPBOARD_FORMAT_TEXT_PLAIN ){
+		buffer = export_objects( clipboard, user_data->items );
+		gtk_selection_data_set( selection_data, selection_data->target, 8, ( const guchar * ) buffer, strlen( buffer ));
+		g_free( buffer );
+
+	} else {
+		gtk_selection_data_set( selection_data, selection_data->target, 8, ( const guchar * ) user_data, sizeof( PrimaryData ));
+	}
 }
 
 static void
-clear_primary_clipboard_callback( GtkClipboard *clipboard, NactClipboardPrimaryData *data )
+clear_primary_clipboard( NactClipboard *clipboard )
 {
-	static const gchar *thisfn = "nact_clipboard_clear_primary_clipboard_callback";
+	static const gchar *thisfn = "nact_clipboard_clear_primary_clipboard";
+	PrimaryData *user_data;
 
-	g_debug( "%s: clipboard=%p, data=%p", thisfn, ( void * ) clipboard, ( void * ) data );
+	g_debug( "%s: clipboard=%p", thisfn, ( void * ) clipboard );
 
-	g_list_foreach( data->items, ( GFunc ) g_object_unref, NULL );
-	g_list_free( data->items );
-	g_free( data );
+	user_data = clipboard->private->primary_data;
+	g_return_if_fail( user_data != NULL );
+
+	g_list_foreach( user_data->items, ( GFunc ) g_object_unref, NULL );
+	g_list_free( user_data->items );
+	user_data->items = NULL;
+	user_data->nb_menus = 0;
+	user_data->nb_actions = 0;
+	user_data->nb_profiles = 0;
+
+	clipboard->private->primary_got = FALSE;
+}
+
+static void
+clear_primary_clipboard_callback( GtkClipboard *gtk_clipboard, NactClipboard *clipboard )
+{
+}
+
+/**
+ * nact_clipboard_dump:
+ * @clipboard: this #NactClipboard instance.
+ *
+ * Dumps the content of the primary clipboard.
+ */
+void
+nact_clipboard_dump( NactClipboard *clipboard )
+{
+	static const gchar *thisfn = "nact_clipboard_dump";
+
+	g_return_if_fail( NACT_IS_CLIPBOARD( clipboard ));
+
+	if( !clipboard->private->dispose_has_run ){
+
+		g_debug( "%s:       window=%p (%s)", thisfn, ( void * ) clipboard->private->window, G_OBJECT_TYPE_NAME( clipboard->private->window ));
+		g_debug( "%s:          dnd=%p", thisfn, ( void * ) clipboard->private->dnd );
+		g_debug( "%s:      primary=%p", thisfn, ( void * ) clipboard->private->primary );
+		g_debug( "%s: primary_data=%p", thisfn, ( void * ) clipboard->private->primary_data );
+
+		if( clipboard->private->primary_data ){
+			dump_primary_clipboard( clipboard );
+		}
+	}
+}
+
+static void
+dump_primary_clipboard( NactClipboard *clipboard )
+{
+	static const gchar *thisfn = "nact_clipboard_dump_primary";
+	PrimaryData *user_data;
+	gchar *mode;
+	GList *it;
+
+	g_return_if_fail( NACT_IS_CLIPBOARD( clipboard ));
+
+	if( !clipboard->private->dispose_has_run ){
+
+		user_data = clipboard->private->primary_data;
+
+		if( user_data ){
+			g_debug( "%s:           user_data->nb_actions=%d", thisfn, user_data->nb_actions );
+			g_debug( "%s:          user_data->nb_profiles=%d", thisfn, user_data->nb_profiles );
+			g_debug( "%s:             user_data->nb_menus=%d", thisfn, user_data->nb_menus );
+			g_debug( "%s:                user_data->items=%p (count=%d)",
+					thisfn,
+					( void * ) user_data->items,
+					user_data->items ? g_list_length( user_data->items ) : 0 );
+			mode = clipboard_mode_to_string( user_data->mode );
+			g_debug( "%s:                 user_data->mode=%d (%s)", thisfn, user_data->mode, mode );
+			g_free( mode );
+			for( it = user_data->items ; it ; it = it->next ){
+				na_object_object_dump( NA_OBJECT( it->data ));
+			}
+		}
+
+		g_debug( "%s: clipboard->private->primary_got=%s", thisfn, clipboard->private->primary_got ? "True":"False" );
+	}
+}
+
+static gchar *
+clipboard_mode_to_string( gint mode )
+{
+	gchar *mode_str;
+
+	switch( mode ){
+		case CLIPBOARD_MODE_CUT:
+			mode_str = g_strdup( "CutMode" );
+			break;
+
+		case CLIPBOARD_MODE_COPY:
+			mode_str = g_strdup( "CopyMode" );
+			break;
+
+		default:
+			mode_str = g_strdup( "unknown mode" );
+			break;
+	}
+
+	return( mode_str );
 }
